# Copyright 2017 NEC, Corp
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from horizon import tables

LOG = logging.getLogger(__name__)


def get_policy_url(obj):
    return reverse('horizon:admin:policies:detail', args=(obj['policy_name'],))


def get_error_table(obj):
    if obj.get('error', 0) == 0:
        return False
    return reverse('horizon:admin:policies:policy_table_detail',
                   args=(obj['policy_name'], 'error'))


def get_warning_table(obj):
    if obj.get('warning', 0) == 0:
        return False
    return reverse('horizon:admin:policies:policy_table_detail',
                   args=(obj['policy_name'], 'warning'))


class MonitoringTable(tables.DataTable):
    errors = tables.Column("error", verbose_name=_("Errors"),
                           link=get_error_table)
    warnings = tables.Column("warning", verbose_name=_("Warnings"),
                             link=get_warning_table)
    policy_name = tables.Column("policy_name",
                                verbose_name=_("violated policy name"),
                                link=get_policy_url)
    policy_description = tables.Column("policy_description",
                                       verbose_name=_("Policy Description"))
    policy_owner_id = tables.Column("policy_owner_id",
                                    verbose_name=_("Policy Owner"))

    class Meta(object):
        name = "monitoring"
        verbose_name = _("Monitoring")
        hidden_title = False



# Copyright 2014 VMware.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.conf.urls import url

from congress_dashboard.datasources import views as data_views
from congress_dashboard.policies.rules import views as rule_views
from congress_dashboard.policies import views


POLICY = r'^(?P<policy_name>[^/]+)/%s$'
POLICYTABLE = r'^(?P<datasource_id>[^/]+)/(?P<policy_table_name>[^/]+)/%s$'


urlpatterns = [
    url(r'^$', views.IndexView.as_view(), name='index'),
    url(r'^create/$', views.CreateView.as_view(), name='create'),
    url(POLICY % 'detail', views.DetailView.as_view(), name='detail'),
    url(POLICYTABLE % 'detail', data_views.DetailView.as_view(),
        name='policy_table_detail'),
    url(POLICY % 'rules/create_raw',
        rule_views.CreateRawView.as_view(), name='create_raw_rule'),
    url(POLICY % 'rules/create',
        rule_views.CreateView.as_view(), name='create_rule'),
]



# Copyright 2017 NEC, Corp
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.core.urlresolvers import reverse
from django.template.defaultfilters import linebreaksbr
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy
from horizon import tables

from congress_dashboard.api import congress


def get_policy_link(datum):
    return reverse('horizon:admin:library:detail', args=(datum['id'],))


class ActivatePolicy(tables.BatchAction):
    name = 'activate_policy'
    verbose_name = _('Activate')
    icon = 'plus'

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Activate Policy",
            u"Activate Policy",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Activated Policy",
            u"Activated Policy",
            count
        )

    def action(self, request, obj_id):
        congress.policy_create(request, {}, obj_id)


class LibraryTable(tables.DataTable):
    id = tables.Column("id", verbose_name=_("Policy ID"), hidden=True,
                       sortable=False)
    name = tables.Column("name", verbose_name=_("Policy Name"),
                         link=get_policy_link)
    desc = tables.WrappingColumn("description", verbose_name=_("Description"),
                                 sortable=False)

    class Meta(object):
        name = "policy_library"
        verbose_name = _("Policy Library")
        row_actions = (ActivatePolicy, )
        hidden_title = True


class LibraryPolicyRulesTable(tables.DataTable):
    name = tables.Column("name", verbose_name=_("Rule Name"),
                         classes=('nowrap-col',))
    rule = tables.Column("rule", verbose_name=_("Rule"),
                         filters=(congress.format_rule, linebreaksbr,))
    comment = tables.WrappingColumn("comment", verbose_name=_("Comment"))

    class Meta(object):
        name = "policy_library_rules"
        verbose_name = _("Policy Rules")
        hidden_title = False



# Copyright 2015 VMware.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages

from congress_dashboard.api import congress


LOG = logging.getLogger(__name__)

POLICY_KIND_CHOICES = (
    ('nonrecursive', _('Nonrecursive')),
    ('action', _('Action')),
    ('database', _('Database')),
    ('materialized', _('Materialized')),
)


class CreatePolicy(forms.SelfHandlingForm):
    name = forms.CharField(max_length=255, label=_("Policy Name"))
    kind = forms.ChoiceField(choices=POLICY_KIND_CHOICES, label=_("Kind"),
                             initial='nonrecursive')
    description = forms.CharField(label=_("Description"), required=False,
                                  widget=forms.Textarea(attrs={'rows': 4}))
    failure_url = 'horizon:admin:policies:index'

    def handle(self, request, data):
        policy_name = data['name']
        policy_description = data.get('description')
        policy_kind = data.pop('kind')
        LOG.info('User %s creating policy "%s" of type %s in tenant %s',
                 request.user.username, policy_name, policy_kind,
                 request.user.tenant_name)
        try:
            params = {
                'name': policy_name,
                'description': policy_description,
                'kind': policy_kind,
            }
            policy = congress.policy_create(request, params)
            msg = _('Created policy "%s"') % policy_name
            LOG.info(msg)
            messages.success(request, msg)
        except Exception as e:
            msg_args = {'policy_name': policy_name, 'error': str(e)}
            msg = _('Failed to create policy "%(policy_name)s": '
                    '%(error)s') % msg_args
            LOG.error(msg)
            messages.error(self.request, msg)
            redirect = reverse(self.failure_url)
            raise exceptions.Http302(redirect)
        return policy



# Copyright 2015 VMware.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages

from congress_dashboard.api import congress


LOG = logging.getLogger(__name__)


class CreateDatasource(forms.SelfHandlingForm):
    name = forms.CharField(max_length=255,
                           label=_("Data Source Name"),
                           help_text='Name of the data source')

    driver = forms.ThemableChoiceField(label=_("Driver"),
                                       help_text='Data Source driver')

    description = forms.CharField(label=_("Description"),
                                  required=False,
                                  help_text='Data Source Description')
    username = forms.CharField(
        max_length=255,
        label=_("UserName"),
        help_text='username to connect to the driver service')

    password = forms.CharField(widget=forms.PasswordInput(render_value=False),
                               label=_('Password'))

    tenant_name = forms.CharField(max_length=255, label=_("Project Name"))

    # TODO(ramineni): support adding lazy tables
    # lazy_tables = forms.CharField(max_length=255, label=_("Lazy Tables"))
    auth_url = forms.URLField(max_length=255, label=_("Keystone Auth URL"))
    poll_time = forms.IntegerField(
        label=_("Poll Interval (in seconds)"),
        help_text='periodic interval congress needs to poll data')

    failure_url = 'horizon:admin:datasources:index'

    @classmethod
    def _instantiate(cls, request, *args, **kwargs):
        return cls(request, *args, **kwargs)

    def __init__(self, request, *args, **kwargs):
        super(CreateDatasource, self).__init__(request, *args, **kwargs)
        driver_choices = [('', _("Select a Driver"))]
        drivers = congress.supported_driver_list(request)
        driver_choices.extend(drivers)
        self.fields['driver'].choices = driver_choices

    def handle(self, request, data):
        datasource_name = data['name']
        datasource_description = data.get('description')
        datasource_driver = data.pop('driver')
        config = {
            'username': data['username'],
            'password': data['password'],
            'tenant_name': data['tenant_name'],
            'auth_url': data['auth_url'],
            'poll_time': data['poll_time']
        }
        try:
            params = {
                'name': datasource_name,
                'driver': datasource_driver,
                'description': datasource_description,
                'config': config
            }
            datasource = congress.create_datasource(request, params)
            msg = _('Created Data Source "%s"') % datasource_name
            LOG.info(msg)
            messages.success(request, msg)
        except Exception as e:
            msg_args = {'datasource_name': datasource_name, 'error': str(e)}
            msg = _('Failed to create data source "%(datasource_name)s": '
                    '%(error)s') % msg_args
            LOG.error(msg)
            messages.error(self.request, msg)
            redirect = reverse(self.failure_url)
            raise exceptions.Http302(redirect)
        return datasource
